namespace BP { namespace String
{
	array<kStr> asciiChars = {
		"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
		"", "", ' ', '!', '"', "", "", '%', '&', '\'', '(', ')', '*', '+', ',', '-', ".", "/", '0', '1', '2', '3', '4', '5', '6',
		'7', '8', '9', ':', ';', '<', '=', '>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
		'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^', '_', '`', 'a', 'b', 'c', 'd', 'e', 'f',
		'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~', ""
	};

	const kStr STRINGLENGTH_KEY = ";L;";
	//------------------------------------------------------------------------------------------------------------------------
	// If s already contains STRINGLENGTH_KEY then this will return an incorrect length
	//------------------------------------------------------------------------------------------------------------------------
	int Length(kStr &in s)
	{
		return (s + STRINGLENGTH_KEY).IndexOf(STRINGLENGTH_KEY);
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool IsEmpty(const kStr &in s)
	{
		return s.Hash() == 0;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// uint to string
	//------------------------------------------------------------------------------------------------------------------------
	kStr UInt(const uint value)
	{
		if (value & BP::Math::UBIT31 == 0)
			return "" + value;
		kStr result;
		array<int> intDigits = BP::Math::Digits(value, false);
		for (uint i = 0; i < intDigits.length(); i++)
			result = result + intDigits[i];
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// destLength: the length of the desired string
	// padChar: the character to put in the padding.
	// Example: PadLeft("hello", 7, "0")  // "00hello"
	//------------------------------------------------------------------------------------------------------------------------
	kStr PadLeft(const kStr &in s, const int destLength, kStr &in padChar)
	{
		kStr result = s;
		int sLength = Length(s);
		for (int i = sLength; i < destLength; i++)
			result = padChar + result;
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	// destLength: the length of the desired string
	// padChar: the character to put in the padding.
	// Example: PadRight("hello", 7, "0")  // "hello00"
	//------------------------------------------------------------------------------------------------------------------------
	kStr PadRight(const kStr &in s, const int destLength, kStr &in padChar)
	{
		kStr result = s;
		int sLength = Length(s);
		for (int i = sLength; i < destLength; i++)
			result += padChar;
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr TimeFormatMMSS(const float time)
	{
		int mins = BP::Math::Min(int(time / 60.0f), 99);
		int sec = int(time % 60.0f);
		return "" + mins + ":" + PadLeft("" + sec, 2, "0");
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<int> SplitInt(const kStr &in s, kStr &in separator)
	{
		array<kStr> entries = Split(s, separator);
		array<int> result;
		for (uint i = 0; i < entries.length(); i++)
			result.insertLast(entries[i].Atoi());
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<float> SplitFloat(const kStr &in s, kStr &in separator)
	{
		array<kStr> entries = Split(s, separator);
		array<float> result;
		for (uint i = 0; i < entries.length(); i++)
			result.insertLast(entries[i].Atof());
		
		return result;
	}
	//------------------------------------------------------------------------------------------------------------------------
	array<kStr> Split(kStr &in s, kStr &in separator)
	{
		array<kStr> results;
		kStr entry;
		int endIndex = Length(s);
		int index = 0;
		while(index >= 0)
		{
			index = IndexEndOf(s, separator, index, endIndex, entry);
			results.insertLast(entry);
		}
		
		return results;
	}
	//------------------------------------------------------------------------------------------------------------------------
	void SplitKeyValues(kStr &in s, kStr &in separator, array<kStr>&out keys, array<kStr>&out values)
	{
		keys.resize(0);
		values.resize(0);
		kStr entry;
		int endIndex = Length(s);
		int index = 0;
		int foundEntryIndex;
		while(index >= 0)
		{
			//Find Key
			index = IndexEndOf(s, separator, index, endIndex, entry);
			foundEntryIndex = keys.find(entry);
			if (foundEntryIndex < 0)
				keys.insertLast(entry);
			
			//Find Value
			index = IndexEndOf(s, separator, index, endIndex, entry);
			if (foundEntryIndex < 0)
				values.insertLast(entry);
			else
				values[foundEntryIndex] = entry;
		}
	}
	//------------------------------------------------------------------------------------------------------------------------
	// Finds the first match in s starting from startIndex to endIndex - 1.
	// Returns the index after the end of the match in s or -1 if not found.
	// Sets the out string to the sub string from startIndex to the start of the match or to endIndex if no match found.
	//------------------------------------------------------------------------------------------------------------------------
	int IndexEndOf(kStr &in s, kStr &in match, const int startIndex, const int endIndex, kStr &out result)
	{
		result = "";
		if (startIndex < 0 || startIndex >= endIndex)
			return -1;
		
		int matchLength = Length(match);
		int matchIndex = 0;
		kStr resultMatch = "";
		for (int i = startIndex; i < endIndex; i++)
		{
			if (s[i] == match[matchIndex])
			{
				matchIndex++;
				if (matchIndex >= matchLength)
					return i + 1;
				resultMatch += asciiChars[s[i]];
			}
			else
			{
				if (matchIndex > 0)
				{
					matchIndex = 0;
					result += resultMatch;
					resultMatch = "";
				}
				result += asciiChars[s[i]];
			}
		}
		if (matchIndex > 0)
			result += resultMatch;

		return -1;
	}
	//------------------------------------------------------------------------------------------------------------------------
	kStr CharAt(kStr &in s, int index)
	{
		int8 c = s[index];
		return c >= 0 ? asciiChars[c] : "";
	}
	//------------------------------------------------------------------------------------------------------------------------
}}
